@@ -28,7 +28,10 @@ class Bliss(DoseDependentModel):
     """
     
     def fit(self, d1, d2, E, drug1_model=None, drug2_model=None, **kwargs):
-        super().fit(d1, d2, E)
+        d1 = np.asarray(d1)
+        d2 = np.asarray(d2)
+        E = np.asarray(E)
+        super().fit(d1,d2,E)
 
         if drug1_model is None:
             mask = np.where(d2==min(d2))
