@@ -13,7 +13,7 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
+from abc import ABC, abstractmethod
 import numpy as np
 from scipy.optimize import curve_fit
 from scipy.stats import norm
@@ -23,7 +23,7 @@ from ..utils import plots
 
 
 
-class ParametricModel:
+class ParametricModel(ABC):
     """Base class for paramterized synergy models, including MuSyC, Zimmer, GPDI, and BRAID.
     """
     def __init__(self):
@@ -62,14 +62,15 @@ class ParametricModel:
         """
         if (self._is_parameterized()):
 
-            n_parameters = len(self.get_parameters())
+            n_parameters = len(self._get_parameters())
 
             self.sum_of_squares_residuals = utils.residual_ss(d1, d2, E, self.E)
             self.r_squared = utils.r_squared(E, self.sum_of_squares_residuals)
             self.aic = utils.AIC(self.sum_of_squares_residuals, n_parameters, len(E))
             self.bic = utils.BIC(self.sum_of_squares_residuals, n_parameters, len(E))
 
-    def get_parameters():
+    @abstractmethod
+    def _get_parameters(self):
         """Returns all of the model's fit parameters
 
         Returns
@@ -77,7 +78,38 @@ class ParametricModel:
         parameters : list or tuple
             Model's parameters
         """
-        return []
+        pass
+
+    @abstractmethod
+    def get_parameters(self, confidence_interval=95):
+        """Returns a dict of the model's parameters.
+
+        When relevant, it will also return meaningful derived parameters. For instance, MuSyC has several parameters for E, but defines a synergy parameter beta as a function of E parameters. Thus, beta will also be included.
+        
+        If the model was fit to data with bootstrap_iterations > 0, this will also return the specified confidence interval.
+        """
+        pass
+
+    @abstractmethod
+    def summary(self, confidence_interval=95, tol=0.01):
+        """Summarizes the model's synergy conclusions.
+
+        For each synergy parameters, determines whether it indicates synergy or antagonism. When the model has been fit with bootstrap_parameters>0, the best fit, lower bound, and upper bound must all agree on synergy or antagonism.
+    
+        Parameters
+        ----------
+        confidence_interval : float, optional (default=95)
+            If the model was fit() with bootstrap_parameters>0, confidence_interval will be used to get the upper and lower bounds. 
+
+        tol : float, optional (default=0.01)
+            Tolerance to determine synergy or antagonism. The parameter must exceed the threshold by at least tol (some parameters, like MuSyC's alpha which is antagonistic from 0 to 1, and synergistic from 1 to inf, will be log-scaled prior to comparison with tol)
+
+        Returns
+        ----------
+        summary : str
+            Tab-separated string. If the model has been bootstrapped, columns are [parameter, value, (lower,upper), synergism/antagonism]. If the model has not been bootstrapped, columns are [parameter, value, synergism/antagonism].
+        """
+        pass
 
     def _internal_fit(self, d, E, use_jacobian, **kwargs):
         """Internal method to fit the model to data (d,E)
@@ -87,11 +119,13 @@ class ParametricModel:
                 popt, pcov = curve_fit(self.fit_function, d, E, bounds=self.bounds, jac=self.jacobian_function, **kwargs)
             else: 
                 popt, pcov = curve_fit(self.fit_function, d, E, bounds=self.bounds, **kwargs)
+            if True in np.isnan(popt):
+                return None
             return self._transform_params_from_fit(popt)
         except:
             return None
 
-    def fit(self, d1, d2, E, drug1_model=None, drug2_model=None, use_jacobian = True, p0=None, bootstrap_iterations=0, bootstrap_confidence_interval=95, **kwargs):
+    def fit(self, d1, d2, E, drug1_model=None, drug2_model=None, use_jacobian = True, p0=None, bootstrap_iterations=0, **kwargs):
         """Fit the model to data.
 
         Parameters
@@ -123,22 +157,20 @@ class ParametricModel:
         kwargs
             kwargs to pass to scipy.optimize.curve_fit()
         """
-        #d1 = np.asarray(d1)
-        #d2 = np.asarray(d2)
 
-        D1 = utils.remove_zeros(d1)
-        D2 = utils.remove_zeros(d2)
+        d1 = np.asarray(d1)
+        d2 = np.asarray(d2)
 
         E = np.asarray(E)
 
-        xdata = np.vstack((D1,D2))
+        xdata = np.vstack((d1,d2))
         
         if 'p0' in kwargs:
             p0 = list(kwargs.get('p0'))
         else:
             p0 = None
         
-        p0 = self._get_initial_guess(D1, D2, E, drug1_model=drug1_model, drug2_model=drug2_model, p0=p0)
+        p0 = self._get_initial_guess(d1, d2, E, drug1_model=drug1_model, drug2_model=drug2_model, p0=p0)
 
         kwargs['p0']=p0
         
@@ -146,15 +178,19 @@ class ParametricModel:
             popt = self._internal_fit(xdata, E, use_jacobian, **kwargs)
 
         if popt is None:
-            self._set_parameters(p0)
+            self._set_parameters(self._transform_params_from_fit(p0))
             self.converged = False
         else:
             self.converged = True
             self._set_parameters(popt)
-            self._score(d1, d2, E)
-            kwargs['p0'] = self._transform_params_to_fit(popt)
-            self._bootstrap_resample(D1, D2, E, use_jacobian, bootstrap_iterations, bootstrap_confidence_interval, **kwargs)
+            n_parameters = len(popt)
+            n_samples = len(d1)
+            if (n_samples - n_parameters - 1 > 0):
+                self._score(d1, d2, E)
+                kwargs['p0'] = self._transform_params_to_fit(popt)
+                self._bootstrap_resample(d1, d2, E, use_jacobian, bootstrap_iterations, **kwargs)
     
+    @abstractmethod
     def E(self, d1, d2):
         """Returns drug effect E at dose d1,d2 for a pre-defined or fitted model.
 
@@ -171,7 +207,7 @@ class ParametricModel:
         effect : array_like
             Evaluate's the model at doses d1 and d2
         """
-        return 0
+        pass
 
     def _is_parameterized(self):
         """Returns False if any parameters are None or nan.
@@ -181,33 +217,38 @@ class ParametricModel:
         is_parameterzed : bool
             True if all of the parameters are set. False if any are None or nan.
         """
-        return not (None in self.get_parameters() or True in np.isnan(np.asarray(self.get_parameters())))
+        return not (None in self._get_parameters() or True in np.isnan(np.asarray(self._get_parameters())))
 
+    @abstractmethod
     def _set_parameters(self, popt):
         """Internal method to set model parameters
         """
         pass
 
+    @abstractmethod
     def _transform_params_from_fit(self, params):
         """Internal method to transform parameterss as needed.
 
         For instance, models that fit logh and logC must transform those to h and C
         """
-        return params
+        pass
 
+    @abstractmethod
     def _transform_params_to_fit(self, params):
         """Internal method to transform parameterss as needed.
 
         For instance, models that fit logh and logC must transform from h and C
         """
-        return params
+        pass
 
+    @abstractmethod
     def _get_initial_guess(self, d1, d2, E, drug1_model=None, drug2_model=None, p0=None):
         """Internal method to format and/or guess p0
         """
-        return p0
+        pass
 
-    def _bootstrap_resample(self, d1, d2, E, use_jacobian, bootstrap_iterations, confidence_interval, **kwargs):
+    
+    def _bootstrap_resample(self, d1, d2, E, use_jacobian, bootstrap_iterations, **kwargs):
         """Internal function to identify confidence intervals for parameters
         """
 
@@ -215,7 +256,7 @@ class ParametricModel:
         if not self.converged: return
 
         n_data_points = len(E)
-        n_parameters = len(self.get_parameters())
+        n_parameters = len(self._get_parameters())
         
         sigma_residuals = np.sqrt(self.sum_of_squares_residuals / (n_data_points - n_parameters))
 
@@ -262,7 +303,7 @@ class ParametricModel:
         ub = 100-lb
         return np.percentile(self.bootstrap_parameters, [lb, ub], axis=0)
 
-    def plot_colormap(self, d1, d2, cmap="viridis", **kwargs):
+    def plot_heatmap(self, d1, d2, cmap="YlGnBu", **kwargs):
         """Plots the model's effect, E(d1, d2) as a heatmap
 
         Parameters
@@ -274,16 +315,16 @@ class ParametricModel:
             Doses of drug 2
         
         kwargs
-            kwargs passed to synergy.utils.plots.plot_colormap()
+            kwargs passed to synergy.utils.plots.plot_heatmap()
         """
         if not self._is_parameterized():
             #raise ModelNotParameterizedError()
             return
         
         E = self.E(d1, d2)
-        plots.plot_colormap(d1, d2, E, cmap=cmap, **kwargs)
+        plots.plot_heatmap(d1, d2, E, cmap=cmap, **kwargs)
 
-    def plot_residual_colormap(self, d1, d2, E, cmap="RdBu", **kwargs):
+    def plot_residual_heatmap(self, d1, d2, E, cmap="RdBu", **kwargs):
         """Plots the model's effect, E(d1, d2) as a heatmap
 
         Parameters
@@ -293,18 +334,24 @@ class ParametricModel:
         
         d2 : array_like
             Doses of drug 2
+
+        E : array_like
+            Observed drug effects
+
+        cmap : string, default="RdBu"
+            Colorscale for the plot
         
         kwargs
-            kwargs passed to synergy.utils.plots.plot_colormap()
+            kwargs passed to synergy.utils.plots.plot_heatmap()
         """
         if not self._is_parameterized():
             #raise ModelNotParameterizedError()
             return
         
         Emodel = self.E(d1, d2)
-        plots.plot_colormap(d1, d2, E-Emodel, cmap=cmap, **kwargs)
+        plots.plot_heatmap(d1, d2, E-Emodel, cmap=cmap, **kwargs)
 
-    def plot_surface_plotly(self, d1, d2, cmap="RdBu", **kwargs):
+    def plot_surface_plotly(self, d1, d2, cmap="YlGnBu", **kwargs):
         """Plots the model's effect, E(d1, d2) as a surface using synergy.utils.plots.plot_surface_plotly()
 
         Parameters
@@ -315,39 +362,15 @@ class ParametricModel:
         d2 : array_like
             Doses of drug 2
         
+        cmap : string, default="viridis"
+            Colorscale for the plot
+
         kwargs
-            kwargs passed to synergy.utils.plots.plot_colormap()
+            kwargs passed to synergy.utils.plots.plot_heatmap()
         """
         if not self._is_parameterized():
             #raise ModelNotParameterizedError()
             return
         
         E = self.E(d1, d2)
-        plots.plot_surface_plotly(d1, d2, E, cmap=cmap, **kwargs)
-
-    def create_fit(d1, d2, E, **kwargs):
-        """Courtesy one-liner to create a model and fit it to data. Appropriate (see model __init__() for details) bounds may be set for curve_fit.
-
-        Parameters
-        ----------
-        d1 : array_like
-            Doses of drug 1
-        
-        d2 : array_like
-            Doses of drug 2
-        
-        E : array_like
-            Dose-response at doses d1 and d2
-
-        X_bounds : tuple, 
-            Bounds for each parameter of the model to be fit. See model.__init__() for specific details.
-        
-        kwargs
-            kwargs to pass sto model.fit()
-
-        Returns
-        ----------
-        model : ParametricModel
-            Synergy model fit to the given data
-        """
-        return
+        plots.plot_surface_plotly(d1, d2, E, cmap=cmap, **kwargs)
\ No newline at end of file
