@@ -17,6 +17,7 @@ import numpy as np
 
 from ..single import Hill, Hill_2P
 from .nonparametric_base import DoseDependentModel
+from .. import utils
 
 class ZIP(DoseDependentModel):
     """The Zero Interaction Potency (ZIP) model (doi: 10.1016/j.csbj.2015.09.001). This model is based on the multiplicative survival principal (i.e., Bliss). All across the dose-response surface, it fits Hill-equations either holding d1==constant or d2==constant.
@@ -40,7 +41,7 @@ class ZIP(DoseDependentModel):
     _C_12 : array_like
         The EC50 of drug 2 obtained by holding D1==constant
     """
-    def __init__(self, E0_bounds=(-np.inf,np.inf), Emax_bounds=(-np.inf,np.inf), h_bounds=(0,np.inf), C_bounds=(0,np.inf)):
+    def __init__(self, E0_bounds=(-np.inf,np.inf), Emax_bounds=(-np.inf,np.inf), h_bounds=(0,np.inf), C_bounds=(0,np.inf), synergyfinder=False):
 
         super().__init__(h1_bounds=h_bounds, h2_bounds=h_bounds, C1_bounds=C_bounds, C2_bounds=C_bounds, E0_bounds=E0_bounds, E1_bounds=Emax_bounds, E2_bounds=Emax_bounds)
 
@@ -49,14 +50,22 @@ class ZIP(DoseDependentModel):
         self.C_bounds = C_bounds
         self.h_bounds = h_bounds
 
+        self.synergyfinder = synergyfinder
+
         self._h_21 = []
         self._h_12 = []
         self._C_21 = []
         self._C_12 = []
+        self._Emax_21 = []
+        self._Emax_12 = []
+        
 
     def fit(self, d1, d2, E, drug1_model=None, drug2_model=None, use_jacobian=True, **kwargs):
     
-        super().fit(d1, d2, E)
+        d1 = np.asarray(d1)
+        d2 = np.asarray(d2)
+        E = np.asarray(E)
+        super().fit(d1,d2,E)
         
         if drug1_model is None:
             mask = np.where(d2==min(d2))
@@ -73,6 +82,7 @@ class ZIP(DoseDependentModel):
         E0_2, Emax_2, h2, C2 = drug2_model.get_parameters()
         E0 = (E0_1+E0_2)/2.
         Emax = (Emax_1+Emax_2)/2.
+
         if (E0 < Emax):
             drug1_model.E0 = Emax_1
             drug2_model.E0 = Emax_2
@@ -82,12 +92,6 @@ class ZIP(DoseDependentModel):
             tmp = E0
             E0 = Emax
             Emax = tmp
-
-
-            # Should I do E = (E-E0)/(E0-Emax) + 1
-            # This puts E on the range 1 to 0.
-            # But it also kills info about drugs that don't reach 0% viability
-
         
         logh1 = np.log(h1)
         logh2 = np.log(h2)
@@ -98,44 +102,127 @@ class ZIP(DoseDependentModel):
         self._h_12 = [] # Hill slope of drug 2, after treated by drug 1
         self._C_21 = [] # EC50 of drug 1, after treated by drug 2
         self._C_12 = [] # EC50 of drug 2, after treated by drug 1
+        self._Emax_21 = []
+        self._Emax_12 = []
         
-        zip_model = Hill_2P(Emax=Emax, h_bounds=self.h_bounds, C_bounds=self.C_bounds)
+        if self.synergyfinder:
+            zip_model = _Hill_3P(Emax_bounds=(-1e-6,1e-6))
+        else:
+            zip_model = _Hill_3P(Emax_bounds=self.Emax_bounds)
+        #zip_model = Hill_2P(Emax=Emax)
+        #zip_model = Hill_2P(Emax=0, h_bounds=self.h_bounds, C_bounds=self.C_bounds)
 
         for D1, D2 in zip(d1, d2):
             # Fix d2==D2, and fit hill for D1
             mask = np.where(d2==D2)
             y2 = drug2_model.E(D2)
             zip_model.E0 = y2
-            zip_model.fit(d1[mask],E[mask], use_jacobian=use_jacobian, p0=[h1,C1])
+            zip_model.fit(d1[mask],E[mask], use_jacobian=use_jacobian, p0=[Emax_1,h1,C1])
             self._h_21.append(zip_model.h)
             self._C_21.append(zip_model.C)
+            self._Emax_21.append(zip_model.Emax)
 
             # Fix d1==D1, and fit hill for D2
             mask = np.where(d1==D1)
             y1 = drug1_model.E(D1)
             zip_model.E0 = y1
-            zip_model.fit(d2[mask],E[mask], use_jacobian=use_jacobian, p0=[h2,C2])
+            zip_model.fit(d2[mask],E[mask], use_jacobian=use_jacobian, p0=[Emax_2,h2,C2])
             self._h_12.append(zip_model.h)
             self._C_12.append(zip_model.C)
+            self._Emax_12.append(zip_model.Emax)
         
         self._h_21 = np.asarray(self._h_21)
         self._h_12 = np.asarray(self._h_12)
         self._C_21 = np.asarray(self._C_21)
         self._C_12 = np.asarray(self._C_12)
+        self._Emax_21 = np.asarray(self._Emax_21)
+        self._Emax_12 = np.asarray(self._Emax_12)
 
-        self.synergy = self._delta_score(d1, d2, E0, Emax, h1, h2, C1, C2, self._h_21, self._h_12, self._C_21, self._C_12)
+        self.synergy = self._delta_score(d1, d2, E0, self.drug1_model.Emax, self.drug2_model.Emax, h1, h2, C1, C2, self._Emax_21, self._Emax_12, self._h_21, self._h_12, self._C_21, self._C_12)
+
+        mask = np.where((d1==0) | (d2==0))
+        self.synergy[mask]=0
+        self.synergy
 
         return self.synergy
 
-    def _delta_score(self, d1, d2, E0, Emax, h1, h2, C1, C2, h_21, h_12, C_21, C_12):
-        dCh1 = (d1/C1)**h1
-        dCh2 = (d2/C2)**h2
-        dCh1_prime = (d1/C_21)**h_21
-        dCh2_prime = (d2/C_12)**h_12
-        
-        AA = ((E0 + Emax*dCh2)/(1.+dCh2) + Emax*dCh1_prime) / (1+dCh1_prime)
-        BB = ((E0 + Emax*dCh1)/(1.+dCh1) + Emax*dCh2_prime) / (1+dCh2_prime)
-        #CC = (E0 + Emax*dCh1)/(1+dCh1) + (E0 + Emax*dCh2)/(1+dCh2) - (E0 + Emax*dCh1)/(1+dCh1)*(E0 + Emax*dCh2)/(1+dCh2) # This form expects E0==0, Emax=1
-        CC = (E0 + Emax*dCh1)/(1+dCh1)*(E0 + Emax*dCh2)/(1+dCh2)
+    def _delta_score(self, d1, d2, E0, E1, E2, h1, h2, C1, C2, Emax_21, Emax_12, h_21, h_12, C_21, C_12):
 
-        return CC - (AA+BB)/2.
+        single_drug_1 = E0 + (E1-E0) * np.power(d1,h1) / (np.power(C1,h1) + np.power(d1,h1))
+
+        single_drug_2 = E0 + (E2-E0) * np.power(d2,h2) / (np.power(C2,h2) + np.power(d2,h2))
+
+        zip_fit_1 = single_drug_2 + (Emax_21-single_drug_2) * np.power(d1,h_21) / (np.power(C_21,h_21) + np.power(d1,h_21))
+
+        zip_fit_2 = single_drug_1 + (Emax_12-single_drug_1) * np.power(d2,h_12) / (np.power(C_12,h_12) + np.power(d2,h_12))
+        
+        zip_fit = (zip_fit_1+zip_fit_2)/2.
+        zip_ind = single_drug_1*single_drug_2
+
+        return zip_ind-zip_fit
+
+
+
+class _Hill_3P(Hill):
+    def __init__(self, E0=1, Emax=0, h=None, C=None, Emax_bounds=(-np.inf, np.inf), h_bounds=(0,np.inf), C_bounds=(0,np.inf)):
+        super().__init__(h=h, C=C, E0=E0, Emax=Emax, Emax_bounds=Emax_bounds, h_bounds=h_bounds, C_bounds=C_bounds)
+
+        self.fit_function = lambda d, Emax, logh, logC: self._model(d, self.E0, Emax, np.exp(logh), np.exp(logC))
+
+        self.jacobian_function = lambda d, Emax, logh, logC: self._model_jacobian(d, Emax, logh, logC)
+
+        self.bounds = tuple(zip(self.Emax_bounds, self.logh_bounds, self.logC_bounds))
+
+    def _model_jacobian(self, d, Emax, logh, logC):
+        dh = d**(np.exp(logh))
+        Ch = (np.exp(logC))**(np.exp(logh))
+        logd = np.log(d)
+        E0 = self.E0
+
+        jEmax = dh/(Ch+dh)
+
+        jC = (E0-Emax)*dh*np.exp(logh+logC)*(np.exp(logC))**(np.exp(logh)-1) / ((Ch+dh)*(Ch+dh))
+
+        jh = (Emax-E0)*dh*np.exp(logh) * ((Ch+dh)*logd - (logC*Ch + logd*dh)) / ((Ch+dh)*(Ch+dh))
+        
+        jac = np.hstack((jEmax.reshape(-1,1), jh.reshape(-1,1), jC.reshape(-1,1)))
+        jac[np.isnan(jac)]=0
+        return jac
+
+    def _get_initial_guess(self, d, E, p0=None):
+
+        if p0 is None:
+            p0 = [np.nanmin(E), 1, np.median(d)]
+            
+        p0 = list(self._transform_params_to_fit(p0))
+        utils.sanitize_initial_guess(p0, self.bounds)
+        
+        return p0
+
+    def get_parameters(self):
+        """Gets the model's parameters
+        
+        Returns
+        ----------
+        parameters : tuple
+            (Emax, h, C)
+        """
+        return (self.Emax, self.h, self.C)
+
+    def _set_parameters(self, popt):
+        Emax, h, C = popt
+        
+        self.Emax = Emax
+        self.h = h
+        self.C = C
+
+    def _transform_params_from_fit(self, params):
+        return params[0], np.exp(params[1]), np.exp(params[2])
+
+    def _transform_params_to_fit(self, params):
+        return params[0], np.log(params[1]), np.log(params[2])
+
+    def __repr__(self):
+        if not self._is_parameterized(): return "Hill_3P()"
+        
+        return "Hill_3P(E0=%0.2f, Emax=%0.2f, h=%0.2f, C=%0.2e)"%(self.E0, self.Emax, self.h, self.C)
\ No newline at end of file
