@@ -68,8 +68,6 @@ class Hill(ParameterizedModel1D):
             self.logh_bounds = (np.log(h_bounds[0]), np.log(h_bounds[1]))
             self.logC_bounds = (np.log(C_bounds[0]), np.log(C_bounds[1]))
 
-        
-
         self.fit_function = lambda d, E0, E1, logh, logC: self._model(d, E0, E1, np.exp(logh), np.exp(logC))
 
         self.jacobian_function = lambda d, E0, E1, logh, logC: self._model_jacobian(d, E0, E1, logh, logC)
@@ -132,8 +130,8 @@ class Hill(ParameterizedModel1D):
         self.C = C
         
     def _model(self, d, E0, Emax, h, C):
-        dh = d**h
-        return E0 + (Emax-E0)*dh/(C**h+dh)
+        dh = np.power(d,h)
+        return E0 + (Emax-E0)*dh/(np.power(C,h)+dh)
 
     def _model_inv(self, E, E0, Emax, h, C):
         d = np.float_power((E0-Emax)/(E-Emax)-1.,1./h)*C
@@ -159,7 +157,9 @@ class Hill(ParameterizedModel1D):
 
         jh = (Emax-E0)*dh*np.exp(logh) * ((Ch+dh)*logd - (logC*Ch + logd*dh)) / ((Ch+dh)*(Ch+dh))
         
-        return np.hstack((jE0.reshape(-1,1), jEmax.reshape(-1,1), jh.reshape(-1,1), jC.reshape(-1,1)))
+        jac = np.hstack((jE0.reshape(-1,1), jEmax.reshape(-1,1), jh.reshape(-1,1), jC.reshape(-1,1)))
+        jac[np.isnan(jac)]=0
+        return jac
 
     def _get_initial_guess(self, d, E, p0=None):
 
@@ -230,7 +230,9 @@ class Hill_2P(Hill):
 
         jh = (Emax-E0)*dh*np.exp(logh) * ((Ch+dh)*logd - (logC*Ch + logd*dh)) / ((Ch+dh)*(Ch+dh))
         
-        return np.hstack((jh.reshape(-1,1), jC.reshape(-1,1)))
+        jac = np.hstack((jh.reshape(-1,1), jC.reshape(-1,1)))
+        jac[np.isnan(jac)]=0
+        return jac
 
     def _get_initial_guess(self, d, E, p0=None):
 
@@ -270,9 +272,9 @@ class Hill_2P(Hill):
         return np.log(params[0]), np.log(params[1])
 
     def __repr__(self):
-        if not self._is_parameterized(): return "Hill()"
+        if not self._is_parameterized(): return "Hill_2P()"
         
-        return "Hill2P(E0=%0.2f, Emax=%0.2f, h=%0.2f, C=%0.2e)"%(self.E0, self.Emax, self.h, self.C)
+        return "Hill_2P(E0=%0.2f, Emax=%0.2f, h=%0.2f, C=%0.2e)"%(self.E0, self.Emax, self.h, self.C)
     
     
 
