@@ -36,12 +36,16 @@ class ParameterizedModel1D:
         """Internal method to fit the model to data (d,E)
         """
         try:
+        #if True:
             if use_jacobian:
                 popt, pcov = curve_fit(self.fit_function, d, E, bounds=self.bounds, jac=self.jacobian_function, **kwargs)
             else: 
                 popt, pcov = curve_fit(self.fit_function, d, E, bounds=self.bounds, **kwargs)
+            if True in np.isnan(popt):
+                return None
             return self._transform_params_from_fit(popt)
         except:
+        #else:
             return None
 
     def _get_initial_guess(self, d, E, p0=None):
@@ -74,8 +78,10 @@ class ParameterizedModel1D:
         kwargs
             kwargs to pass to scipy.optimize.curve_fit()
         """
-        d = utils.remove_zeros(d)
         
+        d = np.asarray(d)
+        E = np.asarray(E)
+
         if 'p0' in kwargs:
             p0 = list(kwargs.get('p0'))
         else:
@@ -87,14 +93,18 @@ class ParameterizedModel1D:
             popt = self._internal_fit(d, E, use_jacobian, **kwargs)
 
         if popt is None:
-            self._set_parameters(p0)
             self.converged = False
+            self._set_parameters(self._transform_params_from_fit(p0))
         else:
             self.converged = True
             self._set_parameters(popt)
-            self._score(d, E)
-            kwargs['p0'] = self._transform_params_to_fit(popt)
-            return self._bootstrap_resample(d, E, use_jacobian, bootstrap_iterations, bootstrap_confidence_interval, **kwargs)
+
+            n_parameters = len(popt)
+            n_samples = len(d)
+            if (n_samples - n_parameters - 1 > 0):
+                self._score(d, E)
+                kwargs['p0'] = self._transform_params_to_fit(popt)
+                self._bootstrap_resample(d, E, use_jacobian, bootstrap_iterations, bootstrap_confidence_interval, **kwargs)
 
     def E(self, d):
         """Evaluate this model at dose d.
